/*     $Id: globalc.h,v 1.18 2009/07/21 16:20:52 marcus_martin Exp $ */
/* 
* MCCCS - Towhee: A Monte Carlo molecular simulation program     *
* Copyright (C) 2003-2005 Marcus G. Martin                       *
* see the file license.gpl for the full license information      *
*                                                                *
* This program is free software; you can redistribute it and/or  *
* modify it under the terms of the GNU General Public License    *
* as published by the Free Software Foundation; either version 2 *
* of the License, or (at your option) any later version.         *
*                                                                *
* This program is distributed in the hope that it will be useful,*
* but WITHOUT ANY WARRANTY; without even the implied warranty of *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  *
* GNU General Public License for more details.                   *
*                                                                *
* You should have received a copy of the GNU General Public      *
* License along with this program; if not, write to the Free     *
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,*
* MA  02111-1307, USA.                                           
*/

/*
  globalc.h
  this file contains all of the information that is at the top 
  of every C routine in Towhee
  last modified 07-02-2009 by M.G. Martin
   generic c includes 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>

/* Include general towhee definitions */
#include "preproc.h"

/* C specific preprocessor derectives */
/* maximum number of timers for keeping track of mpi_wtime */
#define MAXTIMERS 5

#define TRUE 1
#define FALSE 0

/* Structures */
/*
 * user_args contains information about command line arguments passed to
 * towhee.  Populated by parse_args, and passed around as a pointer to be
 * processed where needed.
 */
typedef struct {
  char rex_fn[MAXDIRLENGTH];
  char towhee_input_fn[MAXDIRLENGTH];
  int quiet_mode;
  int parstyle;
  int random_seed;
} user_args;

typedef struct {
  int num_nodes;
  int num_steps;
  int num_rounds;
  int num_swaps;
  float lambda_lj[MAX_FOREIGN_LAMBDA];
  float lambda_c[MAX_FOREIGN_LAMBDA];
} rex_params;

/* 
   template or function declarations 
*/
/* extern void calltest_(); */

extern void towheemainloop_(const int *,const int *, const int *, const int *);
extern void twhquantumloop_();
extern void twhendquantum_();
void timekeeper(int, int, int);
void clearline (FILE *);
void tramonto_control(int, int);
double tramonto_();
void tramontoloop();
void jobfarm(FILE *, int, int);
int get_rex_directory(char*, const char*, int, int, int);
void initialize_wrapper_strings();
void set_towhee_input_file(const char*);
void set_output_dir(const char*);
int populate_directory_initial(char*, const char*, int);
int populate_directory_restart(char*, const char*, int, int);
void initialize_args( user_args*);
void parse_args(int, char**, user_args*);
int get_parstyle(int, int);
void print_usage(const char*);
int read_rex_params(const char*, rex_params*);

void open_towhee_parallel(const char*, FILE **);
void run_rex_server(user_args*, rex_params*);
void run_rex_client(user_args*, rex_params*, int);
int attempt_exchange(float, int, int, double**, int*, rex_params*, FILE*);
int write_lambda_file(const int*, const rex_params*, const char*, int, 
        const char*);
void timestamp(char*);
void **twh_allocate2dMatrix(size_t, int, int);
void *twh_allocateVector(size_t, int);

int initialize_MPI(int, char**);
void do_none(user_args*);
void do_tramonto(user_args*, int);
void do_jobfarm(user_args*, int);
void do_rex(user_args*, int);
void do_lcao(user_args*, int);

void lcao_control(int, int);

void twh_time_array(int, int, double*);
/* 
   USEMPI only  
*/
#ifdef USEMPI
#include <mpi.h>
#endif
/* 
   USETRAMONTO only 
*/
#ifdef USETRAMONTO
extern dftmain(double *);
#endif

/* data storage */
int twh_1ddouble(int, int *, const int, double *);
int twh_2dregdouble(int, int *, int, int, double *);
int twh_1dinteger(int, int *, const int, int *);
int twh_2dreginteger(int, int *, int, int, int *);
int twh_3dreginteger(int, int *, int, int, int, int *);
void twh_acncell_(int *, int *, int *, double *);
void twh_acscell_(int *, int *, int *, double *);
void twh_acncomp_(int *, int *, int *, double *);
void twh_acscomp_(int *, int *, int *, double *);
void twh_acnrot_(int *, int *, int *, double *);
void twh_acsrot_(int *, int *, int *, double *);
void twh_acnswitch_(int *, int *, int *, double *);
void twh_acsswitch_(int *, int *, int *, double *);
void twh_acntraa_(int *, int *, int *, double *);
void twh_acstraa_(int *, int *, int *, double *);
void twh_acntrac_(int *, int *, int *, double *);
void twh_acstrac_(int *, int *, int *, double *);
void twh_bacell_(int *, int *, int *, double *);
void twh_bncell_(int *, int *, int *, double *);
void twh_barot_(int *, int *, int *, double *);
void twh_bnrot_(int *, int *, int *, double *);
void twh_batraa_(int *, int *, int *, double *);
void twh_bntraa_(int *, int *, int *, double *);
void twh_batrac_(int *, int *, int *, double *);
void twh_bntrac_(int *, int *, int *, double *);
void twh_c_matrix_(int *, int *, int *, double *);
void twh_rmcomrot_(int *, int *, int *, double *);
void twh_rmcomtra_(int *, int *, int *, double *);
void twh_rmtraa_(int *, int *, int *, double *);
void twh_rmtrac_(int *, int *, int *, double *);
void twh_rmrot_(int *, int *, int *, double *);
void twh_arbcmofield_(int *, int *, double *);
void twh_comfield_(int *, int *, double *);
void twh_comtempfield_(int *, int *, double *);
void twh_coordfield_(int *, int *, double *);
void twh_coordstorage_(int *, int *, double *);
void twh_coordtemp_(int *, int *, double *);
void twh_cubeletweight_(int *, int *, double *);
void twh_gyration_(int *, int *, double *);
void twh_eam_rho_real_(int *, int *, double *);
void twh_eam_rho_temp_(int *, int *, double *);
void twh_rcmu_(int *, int *, double *);
void twh_rmvol_(int *, int *, double *);
void twh_tmmc_weight_(int *, int *, double *);
void twh_v_semigrand_(int *, int *, double *);
void twh_wrap_foreign_energy_(int *, int *, double *);
void twh_wrap_foreign_lambda_lj_(int *, int *, double *);
void twh_wrap_foreign_lambda_c_(int *, int *, double *);
void twh_acsvol_(int *, int *, int *);
void twh_ewald_kmax_(int *, int *, int *);
void twh_glist_(int *, int *, int *);
void twh_globalpos_(int *, int *, int *);
void twh_growfrom_(int *, int *, int *);
void twh_grownum_(int *, int *, int *);
void twh_growprev_(int *, int *, int *);
void twh_growvalidcount_(int *, int *, int *);
void twh_logical_exist_(int *, int *, int *);
void twh_logical_exsched_(int *, int *, int *);
void twh_logical_moveme_(int *, int *, int *);
void twh_logical_periodic_(int *, int *, int *);
void twh_moltyp_(int *, int *, int *);
void twh_nboxi_(int *, int *, int *);
void twh_parall_(int *, int *, int *, int *);
void twh_chainlist_(int *, int *, int *, int *, int *);
